Package: progressr
Version: 0.19.0
Title: An Inclusive, Unifying API for Progress Updates
Description: A minimal, unifying API for scripts and packages to report progress updates from anywhere including when using parallel processing.  The package is designed such that the developer can to focus on what progress should be reported on without having to worry about how to present it.  The end user has full control of how, where, and when to render these progress updates, e.g. in the terminal using utils::txtProgressBar(), cli::cli_progress_bar(), in a graphical user interface using utils::winProgressBar(), tcltk::tkProgressBar() or shiny::withProgress(), via the speakers using beepr::beep(), or on a file system via the size of a file. Anyone can add additional, customized, progression handlers. The 'progressr' package uses R's condition framework for signaling progress updated. Because of this, progress can be reported from almost anywhere in R, e.g. from classical for and while loops, from map-reduce API:s like the lapply() family of functions, 'purrr', 'plyr', and 'foreach'. It will also work with parallel processing via the 'future' framework, e.g. 'lapply(...) |> futurize()' and 'purrr::map(...) |> futurize()', which uses future.apply::future_lapply() and furrr::future_map() internally. The package is compatible with Shiny applications.
Authors@R: c(person("Henrik", "Bengtsson",
                    role = c("aut", "cre", "cph"),
                    email = "henrikb@braju.com",
                    comment = c(ORCID = "0000-0002-7579-5165")))
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: digest, utils
Suggests: graphics, tcltk, beepr, cli, crayon, pbmcapply, progress,
        purrr, foreach, plyr, doFuture, future, future.apply, furrr,
        ntfy, RPushbullet, rstudioapi, shiny, commonmark, base64enc,
        tools
VignetteBuilder: progressr
Language: en-US
Encoding: UTF-8
URL: https://progressr.futureverse.org,
        https://github.com/futureverse/progressr
BugReports: https://github.com/futureverse/progressr/issues
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-03-31 06:41:07 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7579-5165>)
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2026-03-31 08:10:02 UTC
Built: R 4.6.0; ; 2026-04-25 16:00:55 UTC; unix
