# Generated by roxygen2: do not edit by hand

S3method(conditionMessage,progression)
S3method(length,progressor)
S3method(print,progression)
S3method(print,progression_handler)
S3method(print,progressor)
export(handler_ascii_alert)
export(handler_beepr)
export(handler_brrr)
export(handler_cli)
export(handler_debug)
export(handler_filesize)
export(handler_newline)
export(handler_notifier)
export(handler_ntfy)
export(handler_pbcol)
export(handler_pbmcapply)
export(handler_progress)
export(handler_rpushbullet)
export(handler_rstudio)
export(handler_shiny)
export(handler_slowdown)
export(handler_tkprogressbar)
export(handler_txtprogressbar)
export(handler_void)
export(handler_winprogressbar)
export(handlers)
export(make_progression_handler)
export(progress_aggregator)
export(progress_progressr)
export(progression)
export(progressor)
export(slow_sum)
export(withProgressShiny)
export(with_progress)
export(without_progress)
importFrom(digest,digest)
importFrom(utils,capture.output)
importFrom(utils,file_test)
importFrom(utils,flush.console)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,setTxtProgressBar)
importFrom(utils,str)
importFrom(utils,txtProgressBar)
