#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-with_progress,delay_stdout.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
progressr:::testme("with_progress,delay_stdout")
